@@ -34,7 +34,6 @@
 #include <time.h>
 #include <unistd.h>
 
-#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #define MATEWEATHER_I_KNOW_THIS_IS_UNSTABLE
@@ -558,7 +557,7 @@ _weather_info_fill (WeatherInfo *info,
     info->cb_data = data;
 
     if (!info->session) {
-	info->session = soup_session_async_new ();
+	info->session = soup_session_new_with_options (SOUP_SESSION_TIMEOUT, MATEWEATHER_DEFAULT_TIMEOUT, NULL);
 	soup_session_add_feature_by_type (info->session, SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
     }
 
@@ -580,6 +579,7 @@ weather_info_abort (WeatherInfo *info)
     if (info->session) {
 	soup_session_abort (info->session);
 	info->requests_pending = 0;
+	info->session = NULL;
     }
 }
 
@@ -1657,3 +1657,41 @@ _weather_internal_check (void)
     g_assert (G_N_ELEMENTS (conditions_str) == PHENOMENON_LAST);
     g_assert (G_N_ELEMENTS (conditions_str[0]) == QUALIFIER_LAST);
 }
+
+/**
+ * weather_info_equal:
+ * @info1:  WeatherInfo to compare
+ * @info2:  WeatherInfo to compare
+ *
+ * Returns: gboolean indicating if the two info's are "equal" (i.e. basic
+ *          weather attributes are the same)
+ */
+
+gboolean
+weather_info_equal (WeatherInfo *info1, WeatherInfo *info2)
+{
+    g_return_val_if_fail (info1 != NULL, FALSE);
+    g_return_val_if_fail (info2 != NULL, FALSE);
+    g_return_val_if_fail (info1->valid,  FALSE);
+    g_return_val_if_fail (info2->valid,  FALSE);
+
+    /*
+     * This function is intended to be used to reduce notification spam.
+     * the idea is that unless there's a significant change to either
+     * temp, windspeed, or conditions, there's no need to issue a notification
+     * to the user.
+     */
+
+    if (info1->temp             != info2->temp)
+        return FALSE;
+    if (info1->wind             != info2->wind)
+        return FALSE;
+    if (info1->windspeed        != info2->windspeed)
+        return FALSE;
+    if (info1->cond.significant != info2->cond.significant)
+        return FALSE;
+    if (info1->cond.phenomenon  != info2->cond.phenomenon)
+        return FALSE;
+
+    return TRUE;
+}
